//
//  IPHTMLLocalStorageManager.m
//  LocalStorageTest
//
//  Created by Colden Prime on 8/29/13.
//  Copyright (c) 2013 Apperian. All rights reserved.
//

#import "IPHTMLLocalStorageManager.h"

NSString * const kIPHTMLLocalStorageDidChange = @"kIPHTMLLocalStorageDidChange";

@interface IPHTMLLocalStorageManager () <UIWebViewDelegate>
@property (nonatomic, strong) UIWebView *webView;
@property (nonatomic, assign) BOOL hasLoaded;
@property (nonatomic, strong) NSMutableDictionary *unsetValues;
@end

@implementation IPHTMLLocalStorageManager

+ (IPHTMLLocalStorageManager *)sharedManager {
    static IPHTMLLocalStorageManager *_sharedManager = nil;
    static dispatch_once_t oncePredicate;
    dispatch_once(&oncePredicate, ^{
        _sharedManager = [[IPHTMLLocalStorageManager alloc] init];
    });
    return _sharedManager;
}

- (id)init {
    self = [super init];
    if(self) {
        self.hasLoaded = NO;
        self.unsetValues = [NSMutableDictionary dictionary];

        self.webView = [[UIWebView alloc] initWithFrame:CGRectZero];
        self.webView.delegate = self;

        UIWindow *keyWindow = [UIApplication sharedApplication].keyWindow;
        [keyWindow addSubview:self.webView];
        [keyWindow sendSubviewToBack:self.webView];

        [self.webView loadHTMLString:@"" baseURL:[[NSBundle mainBundle] bundleURL]];
    }
    return self;
}

- (void)hasLoadedSetString:(NSString *)string forKey:(NSString *)aKey {
    [self.webView stringByEvaluatingJavaScriptFromString:[NSString stringWithFormat:
                                                          @"localStorage.setItem(\"%@\", \"%@\");",
                                                          aKey,
                                                          string]];
}

- (void)hasLoadedRemoveStringForKey:(NSString *)aKey {
    [self.webView stringByEvaluatingJavaScriptFromString:[NSString stringWithFormat:
                                                          @"localStorage.removeItem(\"%@\");",
                                                          aKey]];
}

- (void)setString:(NSString *)string forKey:(NSString *)aKey {
    [self setStringsWithDictionary:@{ aKey : string }];
}

- (void)setStringsWithDictionary:(NSDictionary *)dictionary {
    if(self.hasLoaded) {
        for(NSString *aKey in dictionary) {
            NSString *string = [dictionary objectForKey:aKey];
            [self hasLoadedSetString:string forKey:aKey];
        }
        [[NSNotificationCenter defaultCenter] postNotificationName:kIPHTMLLocalStorageDidChange object:nil];
    } else {
        for(NSString *aKey in dictionary) {
            NSString *string = [dictionary objectForKey:aKey];
            [self.unsetValues setObject:string forKey:aKey];
        }
    }
}

- (void)removeStringForKey:(NSString *)aKey {
    [self removeStringWithKeys:@[aKey]];
}

- (void)removeStringWithKeys:(NSArray *)keys {
    if(self.hasLoaded) {
        for(NSString *aKey in keys) {
            [self hasLoadedRemoveStringForKey:aKey];
        }
        [[NSNotificationCenter defaultCenter] postNotificationName:kIPHTMLLocalStorageDidChange object:nil];
    } else {
        for(NSString *aKey in keys) {
            [self.unsetValues setObject:[NSNull null] forKey:aKey];
        }
    }
}

- (NSString *)stringForKey:(NSString *)aKey {
    return [self.webView stringByEvaluatingJavaScriptFromString:[NSString stringWithFormat:@"localStorage.getItem(\"%@\");", aKey]];
}

#pragma mark - UIWebViewDelegate Method

- (void)webViewDidFinishLoad:(UIWebView *)webView {
    self.hasLoaded = YES;

    for(NSString *key in self.unsetValues) {
        id value = [self.unsetValues objectForKey:key];
        if(value == [NSNull null]) {
            [self hasLoadedRemoveStringForKey:key];
        } else {
            [self hasLoadedSetString:value forKey:key];
        }
    }
    [self.unsetValues removeAllObjects];

    [[NSNotificationCenter defaultCenter] postNotificationName:kIPHTMLLocalStorageDidChange object:nil];
}

@end
